/******************************************************************
 *
 * Name:	AL_procs.c
 * 
 * Description:
 * 
 * Author:	Tomasz Stachowiak
 *
 * Date:	20/07/96
 *
 * Change Log:
 *
 ********************************************************************/

#include <stdio.h>
#include <dmedia/audio.h>

#include "AudioParams.h"

#define MAX_USERS 100

ALport micro, speaker;		    /* audio variables */
ALport audio_output_ports[MAX_USERS];
ALconfig conf[MAX_USERS];
ALconfig mic_conf;

/***********************************************************************
 *
 * Audio initializations and closings
 *
 **********************************************************************/
void SetMicGain(long vol)
{
  long buf[4];
  
  buf[0] = AL_LEFT_INPUT_ATTEN;
  buf[2] = AL_RIGHT_INPUT_ATTEN;
  buf[1] = vol;
  buf[3] = vol;
  ALsetparams(AL_DEFAULT_DEVICE, buf, 4);
}


void SetInRate(long rate)
  {
     long buf[2];

     buf[0] = AL_INPUT_RATE;
     buf[1] = rate;

     ALsetparams(AL_DEFAULT_DEVICE, buf, 2);
  }

void SetOutRate(long rate)
  {
     long buf[2];

     buf[0] = AL_OUTPUT_RATE;
     buf[1] = rate;

     ALsetparams(AL_DEFAULT_DEVICE, buf, 2);
  }

void InitAudio()
{
  SetInRate(RATE);
  SetOutRate(RATE);
}

void OpenOutPort(int userno)
{
  char portname[20];

  if ((conf[userno] = ALnewconfig()) == 0)
    fprintf (stderr, "Config allocation error\n");
  if (ALsetchannels(conf[userno], MONO) < 0)
    fprintf (stderr, "Setting channel error\n");
  if (ALsetqueuesize(conf[userno], QSIZE) < 0)
    fprintf (stderr, "Setting queue size error\n");
  if (ALsetwidth(conf[userno], AL_SAMPLE_16) < 0)
    fprintf (stderr, "Setting sample width error\n");

  sprintf(portname, "speaker%d", userno); 
  audio_output_ports[userno] = ALopenport(portname, "w", conf[userno]);

}

void CloseSpeaker()
{
  ALcloseport(speaker);

}

void InitSpeaker(ALport *port)
  {
     if ((*port = ALopenport("speaker", "w", conf)) == 0)
	fprintf (stderr, "Port opening error");
  }

void InitMic()
{  
  if ((mic_conf = ALnewconfig()) == 0)
    fprintf (stderr, "Config allocation error\n");
  if (ALsetchannels(mic_conf, MONO) < 0)
    fprintf (stderr, "Setting channel error\n");
  if (ALsetqueuesize(mic_conf, QSIZE) < 0)
    fprintf (stderr, "Setting queue size error\n");
  if (ALsetwidth(mic_conf, AL_SAMPLE_16) < 0)
    fprintf (stderr, "Setting sample width error\n");

  if ((micro = ALopenport("microphone", "r", mic_conf)) == 0)
    fprintf (stderr, "Port opening error");
}

void CloseMic()
{
  ALcloseport(micro);
}


/*
 * Playing function
 */
void PutFrameToSpeaker(char *buf)
{
  ALwritesamps(speaker, buf, 5*160);
}

/*
 * Reading sound from mic.
 */
void GetFrameFromMic(char *buf)
{
   ALreadsamps(micro, buf, 5*160);
}


void VolumeChange(long vol)
{
  long buf[4];
  
  buf[0] = AL_LEFT_SPEAKER_GAIN;
  buf[2] = AL_RIGHT_SPEAKER_GAIN;
  buf[1] = (long)(2.55*(float)vol);
  buf[3] = (long)(2.55*(float)vol);
  ALsetparams(AL_DEFAULT_DEVICE, buf, 4);
}

long GetVolume()
  {
     long buf[4];

     buf[0] = AL_LEFT_SPEAKER_GAIN;
     buf[2] = AL_RIGHT_SPEAKER_GAIN;

     ALgetparams(AL_DEFAULT_DEVICE, buf, 4);
     return (long)((float)buf[1]/(float)2.55);
  }

